@@ -5,25 +5,37 @@ library("recommenderlab")
 data("MovieLense")
 
 ### test all real rating recommenders
-methods <- unique(sapply(recommenderRegistry$get_entries(
-  dataType="realRatingMatrix"), "[[", "method"))
+context("Test real rating algorithms")
+methods <- unique(sapply(
+  recommenderRegistry$get_entries(dataType = "realRatingMatrix"),
+  "[[",
+  "method"
+))
 
-cat("Available methods for realRatingMatrix:", paste(methods, collapse = ", "))
+if (interactive())
+  cat("Available methods for realRatingMatrix:",
+    paste(methods, collapse = ", "))
 
-MovieLense100 <- MovieLense[rowCounts(MovieLense) > 100,]
-MovieLense100 <- MovieLense[, colCounts(MovieLense100) > 100,]
+MovieLense100 <- MovieLense[rowCounts(MovieLense) > 100, ]
+MovieLense100 <- MovieLense[, colCounts(MovieLense100) > 100, ]
 train <- MovieLense100[1:20]
 test1 <- MovieLense100[101]
 test3 <- MovieLense100[101:103]
 
-for(m in methods) {
-
+for (m in methods) {
   ### skip hybrid recommender
-  if(m == "HYBRID") next
+  if (m == "HYBRID")
+    next
+
+  if (interactive())
+    cat("Algorithm:", m, "\n")
+
+  if (m == "SVD")
+    suppressWarnings(rec <- Recommender(train, method = m))
+  else
+    rec <- Recommender(train, method = m)
+
 
-  context(paste("Algorithm:", m))
-  cat("Algorithm:", m, "\n")
-  rec <- Recommender(train, method = m)
   rec
 
   ### default is top-N list
@@ -47,7 +59,7 @@ for(m in methods) {
 
   ### full rating matrix
   ### RERECOMMEND cannot do it
-  if(m != "RERECOMMEND") {
+  if (m != "RERECOMMEND") {
     pre <- predict(rec, test1, n = 10, type = "ratingMatrix")
     pre
     ### there can be NAs
@@ -59,7 +71,7 @@ for(m in methods) {
   expect_gt(sum(is.na(as(pre, "matrix"))), 0L)
 
   ### RERECOMMEND cannot do it
-  if(m != "RERECOMMEND") {
+  if (m != "RERECOMMEND") {
     pre <- predict(rec, test3, n = 10, type = "ratingMatrix")
     pre
   }
@@ -83,22 +95,31 @@ predict(recom, test1, type = "ratings")
 predict(recom, test3, type = "ratings")
 
 ### test all binary recommenders
-methods <- unique(sapply(recommenderRegistry$get_entries(
-  dataType="binaryRatingMatrix"), "[[", "method"))
+context("Test binary algorithms")
 
-cat("Available methods for binaryRatingMatrix:", paste(methods, collapse = ", "))
+methods <- unique(sapply(
+  recommenderRegistry$get_entries(dataType = "binaryRatingMatrix"),
+  "[[",
+  "method"
+))
+
+if (interactive())
+  cat("Available methods for binaryRatingMatrix:",
+    paste(methods, collapse = ", "))
 
 MovieLense100_bin <- binarize(MovieLense100, minRating = 3)
 train <- MovieLense100_bin[1:50]
 test1 <- MovieLense100_bin[101]
 test3 <- MovieLense100_bin[101:103]
 
-for(m in methods) {
+for (m in methods) {
   ### skip hybrid recommender
-  if(m == "HYBRID") next
+  if (m == "HYBRID")
+    next
 
-  context(paste("Algorithm:", m))
-  cat("Algorithm:", m, "\n")
+
+  if (interactive())
+    cat("Algorithm:", m, "\n")
   rec <- Recommender(train, method = m)
   rec
 
@@ -117,10 +138,13 @@ for(m in methods) {
 
   ## do I get errors?
   pre <- predict(rec, test1, n = 10, type = "ratings")
-  cat("Prediction range (should be [0,1]):\n")
-  print(summary(as.vector(as(pre, "matrix"))))
 
-  if(m != "RERECOMMEND") {
+  if (interactive()) {
+    cat("Prediction range (should be [0,1]):\n")
+    print(summary(as.vector(as(pre, "matrix"))))
+  }
+
+  if (m != "RERECOMMEND") {
     pre <- predict(rec, test1, n = 10, type = "ratingMatrix")
   }
 
@@ -131,7 +155,8 @@ recom <- HybridRecommender(
   Recommender(train, method = "POPULAR"),
   Recommender(train, method = "RANDOM"),
   Recommender(train, method = "AR"),
-  Recommender(train, method = "RERECOMMEND"), ### not implemented for binary data
+  Recommender(train, method = "RERECOMMEND"),
+  ### not implemented for binary data
   weights = c(.25, .25, .25, .25)
 )
 #recom
@@ -141,4 +166,3 @@ predict(recom, test1)
 predict(recom, test3)
 predict(recom, test1, type = "ratings")
 predict(recom, test3, type = "ratings")
-
